/* Projet initialement dveloppe par */
/*      JOVIC Hugo , LY Anthony      */
/* avec la contribution de C. Cerin  */
/* pour la partie affichage          */
/* christophe.cerin@iutv.univ-paris13.fr */

#include<stdio.h>
#include<stdlib.h>
#include<assert.h>
#include<string.h>
#include<stdarg.h>
#include<unistd.h>
#include<time.h>
  typedef enum
{
  BLACK,
  RED,
  GREEN,
  YELLOW,
  BLUE,
  MAGENTA,
  CYAN,
  WHITE,
} COULEUR_TERMINAL;

void clear_terminal ();

int color_printf (COULEUR_TERMINAL fg, COULEUR_TERMINAL bg,
		  const char *format, ...);


typedef struct
{
  int val;
  int initial;
} cellule;

typedef struct
{
  cellule *tab;
  int n;
} grille;

void
clear_terminal ()
{
  printf ("\033[2J");
  printf ("\033[0;0H");
}

int
color_printf1 (COULEUR_TERMINAL fg, COULEUR_TERMINAL bg, const char *format,
	       ...)
{
  int res;
  va_list args;
  printf ("\x1B[1;%02d;%02dm", fg + 30, bg + 40);
  va_start (args, format);
  res = vprintf (format, args);
  va_end (args);
  printf ("\x1B[00m");
  return res;
}

int
color_printf (COULEUR_TERMINAL fg, COULEUR_TERMINAL bg, const char *format,
	      ...)
{
  int res;
  va_list args;
  //printf("\x1B[1;%02d;%02dm", fg + 30, bg + 40);
  va_start (args, format);
  res = vprintf (format, args);
  va_end (args);
  //printf("\x1B[00m");
  return res;
}

grille *
creer_grille (int n)
{
  assert (n == 4 || n == 6 || n == 8);
  grille *g;
  g = malloc (sizeof (grille));
  //on alloue la variable g dynamiquement.
  if (g == NULL)
    exit (0);
  g->n = n;
  //on initialise le champ n grace au pointeur.
  g->tab = malloc (n * n * sizeof (cellule));
  //on alloue la memoire pour chaque case du tableau.
  if (g->tab == NULL)
    exit (0);
  int i;
  for (i = 0; i < n * n; i++)
    {
      g->tab[i].val = -1;
      //pour que la cellule soit vide.
      g->tab[i].initial = 0;
      //pour que ca ne soit pas une cellule initiale.
    }				//la boucle sert permet d 'initialiser une grille vide sans cellule initiale.
  return g;
}

void
detruire_grille (grille * g)
{
  free (g->tab);
  //on desalloue d 'abord la memoire du tableau car si on avait deallouer la variable g d' abord on aurait perdu le tableau l 'interieur du type structure.
  free (g);
}				//autant de free que de malloc.

int
est_indice_valide (grille * g, int indice)
{
  if (indice >= 0 && indice <= g->n - 1)
    //verifier si l 'indice correspond au tableau.
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

int
est_cellule (grille * g, int i, int j)
{
  int res1 = est_indice_valide (g, i);
  //verifier si l 'indice est est bon au niveau des lignes.
  int res2 = est_indice_valide (g, j);
  //verifier si l 'indice est est bon au niveau des colonnes.
  if (res1 == 1 && res2 == 1)
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

int
get_val_cellule (grille * g, int i, int j)
{
  int res = est_cellule (g, i, j);
  assert (res == 1);
  int res2 = g->tab[i * (g->n) + j].val;
  //permet de se deplacer dans le tableau(i = lignes et j = collones)
  return res2;
}

void
set_val_cellule (grille * g, int i, int j, int val)
{
  assert (est_cellule (g, i, j));
  if (val >= -1 && val <= 1)
    {
      g->tab[i * (g->n) + j].val = val;
    }
}

int
est_cellule_initiale (grille * g, int i, int j)
{
  assert (est_cellule (g, i, j));
  int res = g->tab[i * (g->n) + j].initial;
  //on accede au champ initial avec la meme methode que les deux questions precedentes.
  if (res == 1)
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

int
est_cellule_vide (grille * g, int i, int j)
{
  assert (est_cellule (g, i, j));
  int res = get_val_cellule (g, i, j);
  if (res == -1)
    //avec la valeur de la cellule on sait si c 'est une cellule vide ou non. (-1=cellule vide).
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

/**
 * affiche la valeur (WHITE_MAGENTA)
 * @param g : grille
 * @param i : indice de ligne
 * @param j : indice de colonne
 */

void
WHITE_MAGENTA (grille * g, int i, int j)
{

  if (get_val_cellule (g, i, j) == -1)
    {
      color_printf (WHITE, MAGENTA, "       ");
    }
  else
    {
      if (est_cellule_initiale (g, i, j))
	{
	  if (get_val_cellule (g, i, j) == 1)
	    {
	      color_printf (YELLOW, MAGENTA, "   1   ");
	    }
	  else
	    {
	      color_printf (YELLOW, MAGENTA, "   0   ");
	    }
	}
      else
	{
	  if (get_val_cellule (g, i, j) == 1)
	    {
	      color_printf (WHITE, MAGENTA, "   1   ");
	    }
	  else
	    {
	      color_printf (WHITE, MAGENTA, "   0   ");
	    }
	}
    }
}

/**
 * affiche la valeur (WHITE_CYAN)
 * @param g : grille
 * @param i : indice de ligne
 * @param j : indice de colonne
 */

void
WHITE_CYAN (grille * g, int i, int j)
{

  if (get_val_cellule (g, i, j) == -1)
    {
      color_printf (WHITE, CYAN, "       ");
    }
  else
    {
      if (est_cellule_initiale (g, i, j))
	{
	  if (get_val_cellule (g, i, j) == 1)
	    {
	      color_printf (YELLOW, CYAN, "   1   ");
	    }
	  else
	    {
	      color_printf (YELLOW, CYAN, "   0   ");
	    }
	}
      else
	{
	  if (get_val_cellule (g, i, j) == 1)
	    {
	      color_printf (WHITE, CYAN, "   1   ");
	    }
	  else
	    {
	      color_printf (WHITE, CYAN, "   0   ");
	    }
	}
    }
}


/*
 * MAGENTA_CYAN_MAGNETA_CYAN @param g : grille @param i : indice de ligne
 */

void
M_C (grille * g, int i, char c)
{
  int j;

  for (j = 0; j < (g->n); j++)
    {
      if (j % 2 == 0)
	color_printf (WHITE, MAGENTA, "-------");
      else
	color_printf (WHITE, CYAN, "-------");
    }

  printf ("\n%c ", c);
  for (j = 0; j < (g->n); j++)
    {
      if (j % 2 == 0)
	WHITE_MAGENTA (g, i, j);
      else
	WHITE_CYAN (g, i, j);
    }
  printf ("\n  ");
  //for (j = 0; j < (g->n); j++) {
  //if (j % 2 == 0)
  //color_printf(WHITE, MAGENTA, "-------");
  //
  //      else
  //color_printf(WHITE, CYAN, "-------");
  //
  //}
  //printf("\n");
}

/*
 * CYAN_MAGNETA_CYAN_MAGENTA @param g : grille @param i : indice de ligne
 */

void
C_M (grille * g, int i, char c)
{
  int j;

  for (j = 0; j < (g->n); j++)
    {

      if (j % 2 != 0)
	color_printf (WHITE, MAGENTA, "-------");
      else
	color_printf (WHITE, CYAN, "-------");
    }

  printf ("\n%c ", c);
  for (j = 0; j < (g->n); j++)
    {
      if (j % 2 != 0)
	WHITE_MAGENTA (g, i, j);
      else
	WHITE_CYAN (g, i, j);
    }
  printf ("\n  ");
  //for (j = 0; j < (g->n); j++) {
  //if (j % 2 != 0)
  //color_printf(WHITE, MAGENTA, "-------");
  //
  //                      else
  //color_printf(WHITE, CYAN, "-------");
  //
  //}
  //printf("\n");
}

/**
 * Fonction affichant la grille sur le terminal.
 * @param g : pointeur sur la grille que l'on souhaite afficher

 */
void
afficher_grille (grille * g)
{

  //system("clear");

  char c = 'A';
  int i;

  for (i = 0; i < (g->n); i++)
    {
      printf ("     %c ", c);
      c++;
    }
  printf ("\n  ");

  c -= (g->n);

  for (i = 0; i < (g->n); i++)
    {
      if (i % 2 == 0)
	M_C (g, i, c);
      else
	C_M (g, i, c);
      c++;
    }

  for (i = 0; i < (g->n); i++)
    {
      printf ("-------");
    }
  printf ("\n  ");

}

//CODE PARTIE 2
void
rendre_cellule_initiale (grille * g, int i, int j)
{
  assert (est_cellule (g, i, j));
  if ((est_cellule_initiale (g, i, j)) == 0)
    //Si ce n 'est pas une cellule initiale, alors on l' initialise.
    g->tab[g->n * i + j].initial = 1;

}

grille *
initialiser_grille1 (char nom_fichier[])
{
  FILE *f = fopen (nom_fichier,
		   "rt");
  if (f == NULL)
    {
      printf ("Probleme d'ouverture du fichier\n");
      exit (0);
    }
  int n;
  int nb_ini;
  fscanf (f, "%d", &n);
  fscanf (f, "%d", &nb_ini);
  grille *g = creer_grille (n);
  int i;
  int j;
  int val;
  int x;
  for (x = 0; x < nb_ini; x++)
    {
      fscanf (f, "%d %d %d", &i, &j, &val);
      //i pour ligne, j pour colonne, val pour valeur.
      g->tab[g->n * i + j].val = val;
      g->tab[g->n * i + j].initial = 1;
    }
  fclose (f);
  return g;
}

grille *
initialiser_grille (char nom_fichier[])
{
  //declarer des tableaux correspondant aux grilles
  int t1[14] = { 4, 4, 0, 1, 1, 2, 2, 0, 2, 3, 0, 3, 2, 0 };
  int t2[17] = { 4, 5, 0, 1, 0, 0, 3, 0, 2, 0, 1, 2, 1, 1, 3, 2, 0 };
  int t3[16] = { 4, 4, 0, 0, 0, 2, 1, 1, 2, 3, 1, 3, 1, 1 };
  int t4[17] = { 4, 5, 0, 1, 1, 0, 2, 1, 1, 0, 1, 1, 1, 1, 3, 2, 1 };
  int t5[14] = { 4, 4, 0, 3, 0, 1, 0, 1, 1, 1, 1, 2, 0, 1 };
  int t6[30] =
    { 6, 9, 0, 2, 0, 0, 3, 0, 2, 5, 0, 3, 2, 1, 3, 4, 1, 4, 0, 0, 4, 2, 1, 4,
4, 1, 5, 0, 0 };
  int t7[30] =
    { 6, 9, 0, 2, 0, 1, 0, 1, 1, 5, 1, 3, 1, 1, 3, 3, 1, 3, 5, 1, 4, 0, 0, 5,
2, 0, 5, 3, 0 };
  int t8[30] =
    { 6, 9, 0, 3, 0, 0, 4, 0, 1, 0, 1, 1, 3, 0, 2, 4, 1, 3, 1, 1, 3, 4, 1, 3,
5, 1, 5, 0, 1 };
  int t9[32] =
    { 6, 10, 0, 1, 0, 0, 4, 0, 1, 5, 1, 2, 1, 0, 3, 0, 0, 3, 2, 1, 3, 5, 1, 4,
2, 1, 4, 5, 1, 5, 0, 0 };
  int t10[23] =
    { 6, 7, 0, 3, 0, 1, 1, 1, 2, 3, 1, 4, 0, 1, 4, 4, 1, 5, 2, 0, 5, 3, 0 };
  int t11[47] =
    { 8, 15, 0, 0, 0, 0, 5, 1, 1, 0, 0, 1, 3, 0, 1, 6, 0, 2, 2, 1, 2, 5, 1, 3,
1, 1, 3, 7, 0, 5, 1, 0, 5, 6, 0, 5, 7, 0, 7, 2, 1, 7, 5, 0, 7, 6, 0 };
  int t12[50] =
    { 8, 16, 0, 1, 1, 0, 4, 1, 0, 7, 0, 1, 1, 1, 2, 3, 0, 2, 4, 0, 2, 7, 0, 3,
5, 1, 4, 3, 0, 4, 6, 0, 5, 0, 1, 5, 2, 1, 5, 6, 0, 6, 2, 1, 6, 3, 0, 7, 5, 0 };
  int t13[44] =
    { 8, 14, 0, 2, 1, 0, 5, 1, 1, 2, 1, 2, 5, 1, 2, 7, 0, 3, 2, 1, 3, 3, 1, 4,
4, 0, 4, 6, 1, 5, 1, 0, 6, 1, 0, 6, 5, 1, 6, 6, 1, 7, 6, 1 };
  int t14[50] =
    { 8, 16, 0, 1, 1, 0, 6, 0, 0, 7, 0, 1, 4, 1, 3, 2, 1, 3, 4, 1, 3, 5, 1, 4,
2, 1, 4, 4, 0, 4, 6, 1, 6, 1, 0, 6, 2, 0, 6, 5, 1, 7, 1, 0, 7, 4, 1, 7, 7, 0 };
  int t15[38] =
    { 8, 12, 0, 2, 0, 0, 5, 1, 0, 7, 1, 1, 4, 1, 1, 7, 1, 2, 3, 1, 5, 0, 0, 5,
1, 0, 5, 6, 0, 6, 0, 0, 6, 2, 0, 7, 6, 0 };
  int *t;

  //selectionner la grille en fonction de ce que contient le
  // nom du fichier
  //....
  int n = (int) (nom_fichier[9]) - '0';
  int nb_ini = (int) (nom_fichier[17]) - '0';
  //printf("%s*** %d %d ---\n", nom_fichier, n, nb_ini);
  switch (n)
    {
    case 4:
      switch (nb_ini)
	{
	case 1:
	  t = t1;
	  break;
	case 2:
	  t = t2;
	  break;
	case 3:
	  t = t3;
	  break;
	case 4:
	  t = t4;
	  break;
	case 5:
	  t = t5;
	  break;
	}
      break;
    case 6:
      switch (nb_ini)
	{
	case 1:
	  t = t6;
	  break;
	case 2:
	  t = t7;
	  break;
	case 3:
	  t = t8;
	  break;
	case 4:
	  t = t9;
	  break;
	case 5:
	  t = t10;
	  break;
	}
      break;
    case 8:
      switch (nb_ini)
	{
	case 1:
	  t = t11;
	  break;
	case 2:
	  t = t12;
	  break;
	case 3:
	  t = t13;
	  break;
	case 4:
	  t = t14;
	  break;
	case 5:
	  t = t15;
	  break;;
	}
      break;
    }

  n = t[0];
  nb_ini = t[1];

  grille *g = creer_grille (n);
  int i;
  int j;
  int x;
  for (x = 0 + 2; x < (nb_ini * 3) + 2; x = x + 3)
    {
      i = t[x];
      j = t[x + 1];
      g->tab[g->n * i + j].val = t[x + 2];
      g->tab[g->n * i + j].initial = 1;
    }

  return g;
}


int
est_grille_pleine (grille * g)
{
  int i;
  int j;
  int compteur = 0;
  for (i = 0; i < (g->n); i++)
    {
      for (j = 0; j < (g->n); j++)
	{
	  if (est_cellule_vide (g, i, j) == 0)
	    {
	      compteur++;
	    }			//Si la cellule n 'est pas vide, alors on incrémente le compteur.
	}
    }
  if (compteur == (g->n) * (g->n))
    {
      return 1;
    }				//si le compteur est egal au nombre de cellule de la grille alors c 'est une grille pleine.
  else
    {
      return 0;
    }				//sinon elle n 'est pas pleine.
}

int
pas_zero_un_consecutifs (grille * g)
{
  int i;
  int j;
  int compteur = 0;
  for (i = 0; i < (g->n); i++)
    {
      for (j = 0; j < (g->n) - 2; j++)
	{
	  if (get_val_cellule (g, i, j) == get_val_cellule (g, i, j + 1)
	      && get_val_cellule (g, i, j) == get_val_cellule (g, i, j + 2))
	    {
	      compteur++;
	    }			//Si la case est egale a la case suivante et a la case d 'apres on incremente le compteur.(pour ligne)
	}
    }

  for (i = 0; i < (g->n) - 2; i++)
    {
      for (j = 0; j < (g->n); j++)
	{
	  if (get_val_cellule (g, i, j) == get_val_cellule (g, i + 1, j)
	      && get_val_cellule (g, i, j) == get_val_cellule (g, i + 2, j))
	    {
	      compteur++;
	      //meme chose mais au niveau des lignes.
	    }
	}
    }

  if (compteur == 0)
    //Si le compteur n 'a pas change cela veut dire que
    // le test des memes nombres n ' est pas passe et donc il n ' y a pas de 3
    // zero ou 3 un consecutifs.
    return 1;
  else
    return 0;
}

int
meme_nombre_zero_un (grille * g)
{
  int un;
  int zero;
  int res1 = 0;
  int res2 = 0;
  int i;
  int j;
  for (i = 0; i < (g->n); i++)
    {
      un = 0;
      zero = 0;
      for (j = 0; j < (g->n); j++)
	{
	  if (get_val_cellule (g, i, j) == 1)
	    //Si la valeur de la cellule est 1 on incremente le compteur un.
	    un++;

	  else
	    // Sinon on increente le compteur 0 zero
	    zero++;
	}
      if (un == zero)
	//Si il y a autant de un que de zero alors on incremente le compteur res1.
	res1++;
    }

  for (i = 0; i < (g->n); i++)
    {
      un = 0;
      zero = 0;
      for (j = 0; j < (g->n); j++)
	{
	  if (get_val_cellule (g, j, i) == 1)
	    //Meme chose au niveau des colonnes
	    un++;
	  else
	    zero++;

	}


      if (un == zero)
	{
	  res2++;
	}
    }
  if (res1 + res2 == (g->n) * 2)
    //(g->n) * 2 represente le nombre de lignes et de colonnes au total.Si res1 + res2 = (g->n) * 2 alors il y a autant de 1 que de 0
    return 1;

  else
    return 0;

}

int
lignes_colonnes_distinctes (grille * g)
{

  int verif;
  int i;
  int j;
  int k;
  int diff = 0;

  //vertical.La fonction compare la 1er e colonne avec les autres puis la deuxieme avec les autres et ainsi de suite.
  for (k = 0; k < (g->n) - 1; k++)
    {
      for (i = 1 + k; i < (g->n); i++)
	{
	  verif = 0;
	  //on remet a 0 quand on passe a une nouvelle colonne.
	  for (j = 0; j < (g->n); j++)
	    {
	      if (get_val_cellule (g, j, k) == get_val_cellule (g, j, i))
		verif++;
	    }
	  if (verif == (g->n))
	    //Si verif est egal au nombre de cellule de la colonne cela veut dire que les 2 colonnes sont identiques.
	    diff++;
	}
    }


  //horizontal.La fonction compare la 1er e ligne avec les autres puis la deuxieme avec les autres et ainsi de suite.
  for (k = 0; k < (g->n) - 1; k++)
    {
      for (i = 1 + k; i < (g->n); i++)
	{
	  verif = 0;
	  //on remet a 0 quand on passe a une nouvelle ligne.
	  for (j = 0; j < (g->n); j++)
	    {
	      if (get_val_cellule (g, k, j) == get_val_cellule (g, i, j))
		verif++;
	    }
	  if (verif == (g->n))
	    diff++;
	}
    }

  if (diff == 0)
    //Si diff est egal a 0 cela veut dire qu 'il n' y a pas de colonnes ou lignes identiques.
    return 1;
  else
    return 0;

}


int
est_partie_gagnee (grille * g)
{
  if (pas_zero_un_consecutifs (g) == 1 && meme_nombre_zero_un (g) == 1
      && lignes_colonnes_distinctes (g) == 1)
    //Ce sont les regles du Takuzu pour qu 'une partie soit gagnee.
    return 1;
  else
    return 0;
}

//Code partie 3
int
est_mouvement_valide (grille * g, char mouv[], int *ligne, int *colonne,
		      int *val)
{
  *ligne = mouv[0] - 'A';
  *colonne = mouv[1] - 'A';
  if (mouv[2] != '\0')
    *val = mouv[2] - '0';
  else
    *val = -1;

  if (est_cellule (g, *ligne, *colonne) == 1
      && est_cellule_initiale (g, *ligne, *colonne) == 0)
    {
      if (*val == -1)
	{
	  if (get_val_cellule (g, *ligne, *colonne) != -1)
	    set_val_cellule (g, *ligne, *colonne, *val);

	}
      else
	set_val_cellule (g, *ligne, *colonne, *val);

      return 1;
    }
  else
    return 0;


}

void
tour_de_jeu (grille * g)
{
  char mouvement[4] = "AA1";
  int ligne;
  int colonne;
  int val;
  int i;
  do
    {
      printf ("Veuillez jouer \n");
      i = scanf ("%s", mouvement);
      if (i == 0)
	exit (1);
    }
  while (est_mouvement_valide (g, mouvement, &ligne, &colonne, &val) == 0);
}

void
jouer (char ch[])
{
  int i;
  grille *g = initialiser_grille (ch);
  do
    {
      afficher_grille (g);
      for (i = 0; i < 5; i++)
	sleep (1);
      tour_de_jeu (g);
      //printf("\033[H\033[2J");
    }
  while (est_partie_gagnee (g) != 1);
  afficher_grille (g);
  //printf("Veuillez jouer");
  detruire_grille (g);
}

void
choisir_grille (char s[])
{
  char nb_uti = '4';
  int i;
  do
    {
      printf ("Veuillez choisir un nombre entre 4,6 et 8 \n");
      i = scanf ("%c", &nb_uti);
      if (i == 0)
	exit (1);
    }
  while (nb_uti != '4' && nb_uti != '6' && nb_uti != '8');

  s[9] = nb_uti;
  s[17] = (rand () % 5 + 1) + '0';
}


int
main ()
{
  color_printf (GREEN, MAGENTA, "Debut du programme");
  printf ("\n");

  srand (time (NULL));
  char ch[] = "Grilles/GX/grilleX.txt";
  choisir_grille (ch);
  jouer (ch);

  color_printf (YELLOW, CYAN, "Fin du programme");
  printf ("\n");
  return 0;
}
